@@ -95,16 +95,18 @@ class XtdCommentManagerTestCase(ArticleBaseTestCase):
 # In order to test 'save' and '_calculate_thread_data' methods, simulate the
 # following threads, in order of arrival:
 #
-# testcase cmt.id   parent level-0  level-1  level-2
-#  step1     1        -      c1                        <-                 cmt1
-#  step1     2        -      c2                        <-                 cmt2
-#  step2     3        1      --       c3               <-         cmt1 to cmt1
-#  step2     4        1      --       c4               <-         cmt2 to cmt1
-#  step3     5        2      --       c5               <-         cmt1 to cmt2
-#  step4     6        5      --       --        c6     <- cmt1 to cmt1 to cmt2
-#  step4     7        4      --       --        c7     <- cmt1 to cmt2 to cmt1
-#  step5     8        3      --       --        c8     <- cmt1 to cmt1 to cmt1
-#  step5     9        -      c9                        <-                 cmt9
+# testcase cmt.id   parent level-0  level-1  level-2  level-3
+#  step1     1        -      c1                               <-           c1
+#  step1     2        -      c2                               <-           c2
+#  step2     3        1      --       c3                      <-        c3.c1
+#  step2     4        1      --       c4                      <-        c4.c1
+#  step3     5        2      --       c5                      <-        c5.c2
+#  step4     6        5      --       --        c6            <-     c6.c5.c2
+#  step4     7        4      --       --        c7            <-     c7.c4.c1
+#  step5     8        3      --       --        c8            <-     c8.c3.c1
+#  step5     9        -      c9                               <-           c9
+#  step6    10        7                                 c10   <- c10.c7.c4.c1
+#  step6    11        8                                 c11   <- c11.c8.c4.c1
 
 
 def thread_test_step_1(article, model=get_model(), **kwargs):
@@ -116,7 +118,7 @@ def thread_test_step_1(article, model=get_model(), **kwargs):
                          object_pk=article.id,
                          content_object=article,
                          site=site,
-                         comment="comment 1 to article",
+                         comment="c1",
                          submit_date=datetime.now(),
                          **kwargs)
 
@@ -125,7 +127,7 @@ def thread_test_step_1(article, model=get_model(), **kwargs):
                          object_pk=article.id,
                          content_object=article,
                          site=site,
-                         comment="comment 2 to article",
+                         comment="c2",
                          submit_date=datetime.now(),
                          **kwargs)
 
@@ -139,7 +141,7 @@ def thread_test_step_2(article, model=get_model(), **kwargs):
                          object_pk=article.id,
                          content_object=article,
                          site=site,
-                         comment="comment 1 to comment 1",
+                         comment="c3.c1",
                          submit_date=datetime.now(),
                          parent_id=1,
                          **kwargs)
@@ -149,7 +151,7 @@ def thread_test_step_2(article, model=get_model(), **kwargs):
                          object_pk=article.id,
                          content_object=article,
                          site=site,
-                         comment="comment 2 to comment 1",
+                         comment="c4.c1",
                          submit_date=datetime.now(),
                          parent_id=1,
                          **kwargs)
@@ -164,7 +166,7 @@ def thread_test_step_3(article):
                               object_pk=article.id,
                               content_object=article,
                               site=site,
-                              comment="comment 1 to comment 2",
+                              comment="c5.c2",
                               submit_date=datetime.now(),
                               parent_id=2)
 
@@ -178,7 +180,7 @@ def thread_test_step_4(article):
                               object_pk=article.id,
                               content_object=article,
                               site=site,
-                              comment="cmt 1 to cmt 1 to cmt 2",
+                              comment="c6.c5.c2",
                               submit_date=datetime.now(),
                               parent_id=5)
 
@@ -187,7 +189,7 @@ def thread_test_step_4(article):
                               object_pk=article.id,
                               content_object=article,
                               site=site,
-                              comment="cmt 1 to cmt 2 to cmt 1",
+                              comment="c7.c4.c1",
                               submit_date=datetime.now(),
                               parent_id=4)
 
@@ -201,7 +203,7 @@ def thread_test_step_5(article):
                               object_pk=article.id,
                               content_object=article,
                               site=site,
-                              comment="cmt 1 to cmt 1 to cmt 1",
+                              comment="c8.c3.c1",
                               submit_date=datetime.now(),
                               parent_id=3)
 
@@ -210,10 +212,34 @@ def thread_test_step_5(article):
                               object_pk=article.id,
                               content_object=article,
                               site=site,
-                              comment="comment 3 article",
+                              comment="c3",
                               submit_date=datetime.now())
 
 
+def thread_test_step_6(article, model=get_model(), **kwargs):
+    article_ct = ContentType.objects.get(app_label="tests", model="article")
+    if not "site" in kwargs:
+      kwargs["site"] = Site.objects.get(pk=1)
+
+    # post Comment 10 to parent_id 7
+    model.objects.create(content_type=article_ct,
+                         object_pk=article.id,
+                         content_object=article,
+                         comment="c10.c7.c4.c1",
+                         submit_date=datetime.now(),
+                         parent_id=7,
+                         **kwargs)
+
+    # post Comment 11 to parent_id 8
+    model.objects.create(content_type=article_ct,
+                         object_pk=article.id,
+                         content_object=article,
+                         comment="c11.c8.c3.c1",
+                         submit_date=datetime.now(),
+                         parent_id=8,
+                         **kwargs)
+
+
 class BaseThreadStep1TestCase(ArticleBaseTestCase):
     def setUp(self):
         super(BaseThreadStep1TestCase, self).setUp()
@@ -422,6 +448,8 @@ class ThreadStep5TestCase(ArticleBaseTestCase):
         self.assertTrue(self.c8.level == 2 and self.c8.order == 3)
         self.assertEqual(self.c8.nested_count, 0)
 
+    @patch.multiple('django_comments_xtd.conf.settings',
+                    COMMENTS_XTD_MAX_THREAD_LEVEL=2)
     def test_exceed_max_thread_level_raises_exception(self):
         article_ct = ContentType.objects.get(app_label="tests", model="article")
         site = Site.objects.get(pk=1)
@@ -448,6 +476,83 @@ class ThreadStep5TestCase(ArticleBaseTestCase):
         self.assertEqual(cm1.nested_count, 3)
 
 
+class ThreadStep6TestCase(ArticleBaseTestCase):
+    def setUp(self):
+        super(ThreadStep6TestCase, self).setUp()
+        thread_test_step_1(self.article_1)
+        thread_test_step_2(self.article_1)
+        thread_test_step_3(self.article_1)
+        thread_test_step_4(self.article_1)
+        thread_test_step_5(self.article_1)
+        thread_test_step_6(self.article_1)
+
+        (  # content ->    cmt.id  thread_id  parent_id  level  order  nested
+            self.c1,  # ->   1         1          1        0      1      6
+            self.c3,  # ->   3         1          1        1      2      2
+            self.c8,  # ->   8         1          3        2      3      1
+            self.c11, # ->  11         1          8        3      4      0
+            self.c4,  # ->   4         1          1        1      5      2
+            self.c7,  # ->   7         1          4        2      6      1
+            self.c10, # ->  10         1          7        3      7      0
+            self.c2,  # ->   2         2          2        0      1      2
+            self.c5,  # ->   5         2          2        1      2      1
+            self.c6,  # ->   6         2          5        2      3      0
+            self.c9   # ->   9         9          9        0      1      0
+        ) = XtdComment.objects.all()
+
+    def test_threaded_comments_step_6_level_0(self):
+        # comment 1
+        self.assertTrue(self.c1.parent_id == 1 and self.c1.thread_id == 1)
+        self.assertTrue(self.c1.level == 0 and self.c1.order == 1)
+        self.assertEqual(self.c1.nested_count, 6)
+        # comment 2
+        self.assertTrue(self.c2.parent_id == 2 and self.c2.thread_id == 2)
+        self.assertTrue(self.c2.level == 0 and self.c2.order == 1)
+        self.assertEqual(self.c2.nested_count, 2)
+        # comment 9
+        self.assertTrue(self.c9.parent_id == 9 and self.c9.thread_id == 9)
+        self.assertTrue(self.c9.level == 0 and self.c9.order == 1)
+        self.assertEqual(self.c9.nested_count, 0)
+
+    def test_threaded_comments_step_6_level_1(self):
+        # comment 3
+        self.assertTrue(self.c3.parent_id == 1 and self.c3.thread_id == 1)
+        self.assertTrue(self.c3.level == 1 and self.c3.order == 2)
+        self.assertEqual(self.c3.nested_count, 2)
+        # comment 4
+        self.assertTrue(self.c4.parent_id == 1 and self.c4.thread_id == 1)
+        self.assertTrue(self.c4.level == 1 and self.c4.order == 5)
+        self.assertEqual(self.c4.nested_count, 2)
+        # comment 5
+        self.assertTrue(self.c5.parent_id == 2 and self.c5.thread_id == 2)
+        self.assertTrue(self.c5.level == 1 and self.c5.order == 2)
+        self.assertEqual(self.c5.nested_count, 1)
+
+    def test_threaded_comments_step_6_level_2(self):
+        # comment 8
+        self.assertTrue(self.c8.parent_id == 3 and self.c8.thread_id == 1)
+        self.assertTrue(self.c8.level == 2 and self.c8.order == 3)
+        self.assertEqual(self.c8.nested_count, 1)
+        # comment 7
+        self.assertTrue(self.c7.parent_id == 4 and self.c7.thread_id == 1)
+        self.assertTrue(self.c7.level == 2 and self.c7.order == 6)
+        self.assertEqual(self.c7.nested_count, 1)
+        # comment 6
+        self.assertTrue(self.c6.parent_id == 5 and self.c6.thread_id == 2)
+        self.assertTrue(self.c6.level == 2 and self.c6.order == 3)
+        self.assertEqual(self.c6.nested_count, 0)
+
+    def test_threaded_comments_step_6_level_3(self):
+        # comment 10
+        self.assertTrue(self.c10.parent_id == 7 and self.c10.thread_id == 1)
+        self.assertTrue(self.c10.level == 3 and self.c10.order == 7)
+        self.assertEqual(self.c10.nested_count, 0)
+        # comment 11
+        self.assertTrue(self.c11.parent_id == 8 and self.c11.thread_id == 1)
+        self.assertTrue(self.c11.level == 3 and self.c11.order == 4)
+        self.assertEqual(self.c11.nested_count, 0)
+
+
 def add_comment_to_diary_entry(diary_entry):
     diary_ct = ContentType.objects.get(app_label="tests", model="diary")
     site = Site.objects.get(pk=1)
